module Pavlok
  class Stimulus
    attr_reader :stimulus, @errors
    
    VALID_RANGES = {
      "zap": 1..100,
      "vibration": 1..100,
      "beep": 1..100,
    }
  
    def initialize(kind, value, message)
      validate
      @kind, @value, @message = kind, value, message
      @stimulus = OpenStruct.new(value: value, kind: kind, message: message, data: { reason: message })
    end
    
    def self.make_zap(intensity:, message:)
      self.new("zap", transform_to_api(intensity), message)
    end
    
    def self.make_vibrate(intensity:, message:)
      self.new("vibration", transform_to_api(intensity), message)
    end
    
    def self.make_beep(tune:, message:)
      self.new("beep", transform_to_api(tune), message)
    end
    
    private
    
      def validate
        raise InvalidStimulusTypeError if !VALID_RANGES.keys.include?(@kind)
        raise InvalidStimulusValueError if !VALID_RANGES[@kind].include?(@value)
      end
  
      def transform_to_api(value)
        ((value.to_f / 255) * 100).ceil.to_i
      end
      
  end
end